@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2013-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,6 +10,7 @@ import socket
 import sys
 
 from xpra.os_util import WIN32
+from xpra.net.common import FLUSH_HEADER
 from xpra.log import Logger
 
 log = Logger("network", "util")
@@ -28,6 +29,7 @@ def import_netifaces() -> object:
         except ImportError:
             _netifaces = False
             log.warn("Warning: the python netifaces package is missing")
+            log.warn(" some networking functionality will be unavailable")
     return _netifaces
 
 iface_ipmasks = {}
@@ -106,7 +108,6 @@ def get_bind_IPs():
     return bind_IPs
 
 def do_get_bind_IPs():
-    global iface_ipmasks
     ips = []
     netifaces = import_netifaces()
     assert netifaces
@@ -143,12 +144,14 @@ def do_get_bind_ifacemask(iface):
                 mask = address['netmask']
                 if addr!= '::1' and addr != '0.0.0.0' and addr.find("%")<0:
                     try:
-                        socket.inet_aton(addr)
-                        ipmasks.append((addr,mask))
+                        fam = socket.AF_INET6 if len(addr.split(":"))>2 else socket.AF_INET
+                        b = socket.inet_pton(fam, addr)
+                        log(f"socket.inet_pton(AF_INET%s, {addr}={b}", "6" if fam==socket.AF_INET6 else "")
+                        ipmasks.append((addr, mask))
                     except Exception as e:
-                        log("do_get_bind_ifacemask(%s)", iface, exc_info=True)
-                        log.error("Error converting address '%s' to binary, for interface %s", addr, iface)
-                        log.error(" %s", e)
+                        log(f"do_get_bind_ifacemask({iface})", exc_info=True)
+                        log.error(f"Error converting address {addr!r} with mask {mask!r} to binary for interface {iface}")
+                        log.error(f" {e}")
     log("do_get_bind_ifacemask(%s)=%s", iface, ipmasks)
     return ipmasks
 
@@ -175,7 +178,7 @@ def get_iface(ip) -> str:
             assert len(v)>0
         except Exception as e:
             log("socket.getaddrinfo(%s, None)", ip, exc_info=True)
-            log.error("Error: cannot revolve '%s'", ip)
+            log.error(f"Error revolving {ip!r}: {e}")
             return None
         for i, x in enumerate(v):
             family, socktype, proto, canonname, sockaddr = x
@@ -191,29 +194,44 @@ def get_iface(ip) -> str:
 
     best_match = None
     get_bind_IPs()
-    for (iface, ipmasks) in iface_ipmasks.items():
-        for (test_ip,mask) in ipmasks:
+    for iface, ipmasks in iface_ipmasks.items():
+        for test_ip, mask in ipmasks:
             if test_ip == ip:
                 #exact match
                 log("get_iface(%s)=%s", iface, ip)
                 return iface
-            test_ip_parts = test_ip.split(".")
-            mask_parts = mask.split(".")
-            if len(test_ip_parts)!=4 or len(mask_parts)!=4:
-                log.error("incorrect ip or mask: %s/%s", test_ip, mask)
+            ipv6 = len(test_ip.split(":"))>2
+            if ipv6:
+                test_ip_parts = test_ip.split("/")[0].split(":")
+                mask_parts = mask.split("/")[0].split(":")
+            else:
+                test_ip_parts = test_ip.split(".")
+                mask_parts = mask.split(".")
+                if len(test_ip_parts)!=4 or len(mask_parts)!=4:
+                    log.error(f"Error: incorrect IP {test_ip} or mask {mask}")
             match = True
             try:
-                for i in (0,1,2,3):
-                    mask_part = int(mask_parts[i])
-                    ip_part = int(ip_parts[i]) & mask_part
-                    test_ip_part = int(test_ip_parts[i]) & mask_part
+                for i, ip_part_str in enumerate(test_ip_parts):
+                    if i>=len(mask_parts):
+                        #end of the mask
+                        break
+                    try:
+                        mask_part = int(mask_parts[i] or 0)
+                    except ValueError:
+                        mask_part = int(mask_parts[i], 16)
+                    try:
+                        ip_part = int(ip_part_str or 0)
+                    except ValueError:
+                        ip_part = int(ip_part_str, 16)
+                    test_ip_part = ip_part & mask_part
                     if ip_part!=test_ip_part:
                         match = False
                         break
                 if match:
                     best_match = iface
             except Exception as e:
-                log.error("error parsing ip (%s) or its mask (%s): %s", test_ip, mask, e)
+                log("ip parsing error", exc_info=True)
+                log.error(f"Error parsing IP {test_ip!r} or its mask {mask!r}: {e}")
     log("get_iface(%s)=%s", ip, best_match)
     return best_match
 
@@ -252,59 +270,68 @@ def get_net_sys_config():
             def addproc(procpath, subsystem, name, conv=stripnl):
                 assert name
                 try:
-                    with open(procpath) as f:
+                    with open(procpath, mode="r", encoding="latin1") as f:
                         data = f.read()
                         subdict = net_sys_config.setdefault(subsystem, {})
                         if name.find("/")>0:
                             sub, name = name.split("/", 1)
                             subdict = subdict.setdefault(sub, {})
                         for sub in ("ip", "tcp", "ipfrag", "icmp", "igmp"):
-                            if name.startswith("%s_" % sub):
+                            if name.startswith(f"{sub}_"):
                                 name = name[len(sub)+1:]
                                 subdict = subdict.setdefault(sub, {})
                                 break
                         subdict[name] = conv(data)
                 except Exception as e:
                     log("cannot read '%s': %s", procpath, e)
-            for k in ("netdev_max_backlog", "optmem_max", "rmem_default", "rmem_max", "wmem_default", "wmem_max", "max_skb_frags",
-                    "busy_poll", "busy_read", "somaxconn"):
-                addproc("/proc/sys/net/core/%s" % k,     "core", k, int)
+            for k in ("netdev_max_backlog", "optmem_max",
+                      "rmem_default", "rmem_max", "wmem_default", "wmem_max", "max_skb_frags",
+                      "busy_poll", "busy_read", "somaxconn",
+                      ):
+                addproc(f"/proc/sys/net/core/{k}",  "core", k, int)
             for k in ("default_qdisc", ):
-                addproc("/proc/sys/net/core/%s" % k,     "core", k)
+                addproc(f"/proc/sys/net/core/{k}",  "core", k)
             for k in ("max_dgram_qlen", ):
-                addproc("/proc/sys/net/unix/%s" % k,     "unix", k, int)
-            for k in ("ip_forward", "ip_forward_use_pmtu", "tcp_abort_on_overflow", "fwmark_reflect", "tcp_autocorking", "tcp_dsack",
-                    "tcp_ecn_fallback", "tcp_fack",
-                    #"tcp_l3mdev_accept",
-                    "tcp_low_latency", "tcp_no_metrics_save", "tcp_recovery", "tcp_retrans_collapse", "tcp_timestamps",
-                    "tcp_workaround_signed_windows", "tcp_thin_linear_timeouts", "tcp_thin_dupack", "ip_nonlocal_bind",
-                    "ip_dynaddr", "ip_early_demux", "icmp_echo_ignore_all", "icmp_echo_ignore_broadcasts",
-                    ):
-                addproc("/proc/sys/net/ipv4/%s" % k,     "ipv4", k, bool)
-            for k in ("tcp_allowed_congestion_control", "tcp_available_congestion_control", "tcp_congestion_control", "tcp_early_retrans",
-                    "tcp_moderate_rcvbuf", "tcp_rfc1337", "tcp_sack", "tcp_slow_start_after_idle", "tcp_stdurg",
-                    "tcp_syncookies", "tcp_tw_recycle", "tcp_tw_reuse", "tcp_window_scaling",
-                    "icmp_ignore_bogus_error_responses", "icmp_errors_use_inbound_ifaddr"):
-                addproc("/proc/sys/net/ipv4/%s" % k,     "ipv4", k)
+                addproc(f"/proc/sys/net/unix/{k}",  "unix", k, int)
+            for k in (
+                "ip_forward", "ip_forward_use_pmtu",
+                "tcp_abort_on_overflow", "fwmark_reflect", "tcp_autocorking", "tcp_dsack",
+                "tcp_ecn_fallback", "tcp_fack",
+                #"tcp_l3mdev_accept",
+                "tcp_low_latency", "tcp_no_metrics_save", "tcp_recovery", "tcp_retrans_collapse", "tcp_timestamps",
+                "tcp_workaround_signed_windows", "tcp_thin_linear_timeouts", "tcp_thin_dupack", "ip_nonlocal_bind",
+                "ip_dynaddr", "ip_early_demux", "icmp_echo_ignore_all", "icmp_echo_ignore_broadcasts",
+                ):
+                addproc(f"/proc/sys/net/ipv4/{k}",  "ipv4", k, bool)
+            for k in (
+                "tcp_allowed_congestion_control", "tcp_available_congestion_control",
+                "tcp_congestion_control", "tcp_early_retrans",
+                "tcp_moderate_rcvbuf", "tcp_rfc1337", "tcp_sack", "tcp_slow_start_after_idle", "tcp_stdurg",
+                "tcp_syncookies", "tcp_tw_recycle", "tcp_tw_reuse", "tcp_window_scaling",
+                "icmp_ignore_bogus_error_responses", "icmp_errors_use_inbound_ifaddr",
+                ):
+                addproc(f"/proc/sys/net/ipv4/{k}",  "ipv4", k)
             def parsenums(v):
                 return tuple(int(x.strip()) for x in v.split("\t") if len(x.strip())>0)
             for k in ("tcp_mem", "tcp_rmem", "tcp_wmem", "ip_local_port_range", "ip_local_reserved_ports", ):
-                addproc("/proc/sys/net/ipv4/%s" % k,     "ipv4", k, parsenums)
-            for k in ("ip_default_ttl", "ip_no_pmtu_disc", "route/min_pmtu",
-                    "route/mtu_expires", "route/min_adv_mss",
-                    "ipfrag_high_thresh", "ipfrag_low_thresh", "ipfrag_time", "ipfrag_max_dist",
-                    "tcp_adv_win_scale", "tcp_app_win", "tcp_base_mss", "tcp_ecn", "tcp_fin_timeout", "tcp_frto",
-                    "tcp_invalid_ratelimit", "tcp_keepalive_time", "tcp_keepalive_probes", "tcp_keepalive_intvl",
-                    "tcp_max_orphans", "tcp_max_syn_backlog", "tcp_max_tw_buckets",
-                    "tcp_min_rtt_wlen", "tcp_mtu_probing", "tcp_probe_interval", "tcp_probe_threshold", "tcp_orphan_retries",
-                    "tcp_reordering", "tcp_max_reordering", "tcp_retries1", "tcp_retries2", "tcp_synack_retries",
-                    "tcp_fastopen", "tcp_syn_retries", "tcp_min_tso_segs", "tcp_pacing_ss_ratio",
-                    "tcp_pacing_ca_ratio", "tcp_tso_win_divisor", "tcp_notsent_lowat",
-                    "tcp_limit_output_bytes", "tcp_challenge_ack_limit",
-                    "icmp_ratelimit", "icmp_msgs_per_sec", "icmp_msgs_burst", "icmp_ratemask",
-                    "igmp_max_memberships", "igmp_max_msf", "igmp_qrv",
-                    ):
-                addproc("/proc/sys/net/ipv4/%s" % k,     "ipv4", k, int)
+                addproc(f"/proc/sys/net/ipv4/{k}",  "ipv4", k, parsenums)
+            for k in (
+                "ip_default_ttl", "ip_no_pmtu_disc", "route/min_pmtu",
+                "route/mtu_expires", "route/min_adv_mss",
+                "ipfrag_high_thresh", "ipfrag_low_thresh", "ipfrag_time", "ipfrag_max_dist",
+                "tcp_adv_win_scale", "tcp_app_win", "tcp_base_mss", "tcp_ecn", "tcp_fin_timeout", "tcp_frto",
+                "tcp_invalid_ratelimit", "tcp_keepalive_time", "tcp_keepalive_probes", "tcp_keepalive_intvl",
+                "tcp_max_orphans", "tcp_max_syn_backlog", "tcp_max_tw_buckets",
+                "tcp_min_rtt_wlen", "tcp_mtu_probing", "tcp_probe_interval",
+                "tcp_probe_threshold", "tcp_orphan_retries",
+                "tcp_reordering", "tcp_max_reordering", "tcp_retries1", "tcp_retries2", "tcp_synack_retries",
+                "tcp_fastopen", "tcp_syn_retries", "tcp_min_tso_segs", "tcp_pacing_ss_ratio",
+                "tcp_pacing_ca_ratio", "tcp_tso_win_divisor", "tcp_notsent_lowat",
+                "tcp_limit_output_bytes", "tcp_challenge_ack_limit",
+                "icmp_ratelimit", "icmp_msgs_per_sec", "icmp_msgs_burst", "icmp_ratemask",
+                "igmp_max_memberships", "igmp_max_msf", "igmp_qrv",
+                ):
+                addproc(f"/proc/sys/net/ipv4/{k}",  "ipv4", k, int)
     return net_sys_config
 
 def get_net_config() -> dict:
@@ -360,7 +387,7 @@ def get_ssl_info(show_constants=False) -> dict:
     return info
 
 
-def get_network_caps() -> dict:
+def get_network_caps(full=True) -> dict:
     from xpra.net.digest import get_digests
     from xpra.net.crypto import get_crypto_caps
     from xpra.net.compression import get_enabled_compressors, get_compression_caps
@@ -374,12 +401,10 @@ def get_network_caps() -> dict:
                 "compressors"           : get_enabled_compressors(),
                 "encoders"              : get_enabled_encoders(),
                }
-    from xpra.net.common import FLUSH_HEADER
-    if FLUSH_HEADER:
-        caps["flush"] = True
-    caps.update(get_crypto_caps())
-    caps.update(get_compression_caps())
-    caps.update(get_packet_encoding_caps())
+    caps["flush"] = FLUSH_HEADER
+    caps.update(get_crypto_caps(full))
+    caps.update(get_compression_caps(full))
+    caps.update(get_packet_encoding_caps(full))
     return caps
 
 
